#include "common.h"

// extract an integer for para (eg, -a, -t)
// mode = 1 forces para to exist in args, otherwise terminate
// mode = 0 allows para not to exist in args, and then return -1
int getParaInt(int *result, int argc, char *args[], char *para, int mode)
{
	int i;
	for(i=1; i<argc; i++)
	{
		if(strcmp(args[i], para) == 0)
		{
			*result = atoi(args[i+1]);
			return 1;
		}
	}

	if(mode == 1)
	{
		cout<<"Missing parameter for "<<para<<endl;
		exit(1);
	}

	return 0;
}


// extract an integer for para (eg, -a, -t)
// mode = 1 forces para to exist in args, otherwise terminate
// mode = 0 allows para not to exist in args, and then return -1
int getParaLong(long *result, int argc, char *args[], char *para, int mode)
{
	int i;
	for(i=1; i<argc; i++)
	{
		if(strcmp(args[i], para) == 0)
		{
			*result = atol(args[i+1]);
			return 1;
		}
	}

	if(mode == 1)
	{
		cout<<"Missing parameter for "<<para<<endl;
		exit(1);
	}

	return 0;
}


// extract an floating point number for para (eg, -r, -c)
// mode = 1 forces para to exist in args, otherwise terminate
// mode = 0 allows para not to exist in args, and then return -1
int getParaFloat(double *result, int argc, char *args[], char *para, int mode)
{
	int i;
	for(i=1; i<argc; i++)
	{
		if(strcmp(args[i], para) == 0)
		{
			*result = atof(args[i+1]);
			return 1;
		}
	}
	if(mode == 1)
	{
		cout<<"Missing parameter for "<<para<<endl;
		exit(1);
	}

	return 0;
}


// extract an floating point number for para (NOT used in this project)
// mode = 1 forces para to exist in args, otherwise terminate
// mode = 0 allows para not to exist in args, and then return -1
int getParaString(char **result, int argc, char *args[], char *para, int mode)
{
	int i;
	for(i=1; i<argc; i++)
	{
		if(strcmp(args[i], para) == 0)
		{
			if(*result == NULL)
				*result = new char[strlen(args[i+1]) + 1];
			else 
			{
				delete [](*result);
				*result = new char[strlen(args[i+1]) + 1];
			}

			strcpy(*result, args[i+1]);
			return 1;
		}
	}

	if(mode == 1)
	{
		cout<<"Missing parameter for "<<para<<endl;
		exit(1);
	}

	return -1;
}


void compressDir(char *filename)
{
	char *target;
	int length = strlen(filename), i, flag = 1, j=0;
	target = new char[length + 1];

	for(i=0; i<length; i++)
	{
		if(filename[i] == '/')
		{
			if(flag == 0)
				flag = 1;
			else
				continue;
		}
		else
			flag = 0;

		target[j++] = filename[i];
	}

	target[j] = '\0';
	strcpy(filename, target);

	delete []target;
}


/*
void readPara(int argc, char *args[], double *data, char *sigmaString, char *directory)
{
	double dresult;
	int iresult, flag, i;
	char *temp;

	for(i=0; i<20; i++)
		data[i] = 0;

	iresult = getParaInt(argc, args, "-t", &flag);
	if(flag == 1)
		data[0] = iresult;		
	else
	{
		cout<<"You must specify -t.\n";
		exit(1);
	}

	temp = getParaString(argc, args, "-dir", &flag);
	if(flag == 0 && data[0] > 0)
	{
		cout<<"You must specify -dir.\n";
		exit(1);
	}
	if(temp)
	{
		strcpy(directory, temp);
		i = strlen(directory) - 1;
		while(i>=0 && directory[i] == '/')
		{
			directory[i] = '\0';
			i--;
		}
	}
	
	temp = getParaString(argc, args, "-sig", &flag);
	strcpy(sigmaString, temp);

	if(flag == 1)	
		data[1] = atof(sigmaString);
	else
	{
		cout<<"You must specify -sig.\n";
		exit(1);
	}

	dresult = getParaFloat(argc, args, "-C", &flag);
	if(flag == 1)
		data[2] = dresult;
	else if(data[0] == 1 || data[0] == 2)
	{
		cout<<"You must specify -C for type "<<data[0]<<endl;
		exit(1);
	}

	dresult = getParaInt(argc, args, "-l", &flag);
	if(flag == 1)
		data[3] = dresult;
	else 
		data[3] = -1;	

	data[4] = -1;
	if(data[0] == 2)
	{
		dresult = getParaFloat(argc, args, "-f", &flag);
		if(flag == 1)
			data[4] = dresult;
	}

	data[5] = data[6] = 10; 
	data[7] = 1;
	if(data[0] == 3)
	{
		dresult = getParaFloat(argc, args, "-C1", &flag);
		if(flag == 1)
			data[5] = dresult;
	
		dresult = getParaFloat(argc, args, "-C2", &flag);
		if(flag == 1)
			data[6] = dresult;

		dresult = getParaFloat(argc, args, "-ep", &flag);
		if(flag == 1)
			data[7] = dresult;
	}

	dresult = getParaInt(argc, args, "-n", &flag);
	if(flag == 1)
		data[8] = dresult;
	else
		data[8] = -1;
}
*/

int exist(char *filename)
{
	FILE *input;
	input = fopen(filename, "r");
	if( input == NULL)
		return 0;
	fclose(input);
	return 1;
}


void *checkMalloc(long int size)
{
	void *ptr;
	ptr=(void *)malloc(size);
	if(!ptr) { 
		perror ("Out of memory!\n"); 
		exit (1); 
	}
	return(ptr);
}


int Partition(int *values, int lower, int upper)
{
	int pivot  = values[upper];
	int i, j;
	int temp;

	i = lower - 1;

	for(j=lower; j<upper; j++)
	{
		if(values[j] <= pivot)
		{
			i ++;
			temp = values[i];
			values[i] = values[j];
			values[j] = temp;
		}
	}

	temp = values[i+1];
	values[i+1] = values[upper];
	values[upper] = temp;

	return i+1;
}


int Partition(double *values, int lower, int upper)
{
	double pivot  = values[upper], temp;
	int i, j;	

	i = lower - 1;

	for(j=lower; j<upper; j++)
	{
		if(values[j] <= pivot)
		{
			i ++;
			temp = values[i];
			values[i] = values[j];
			values[j] = temp;
		}
	}

	temp = values[i+1];
	values[i+1] = values[upper];
	values[upper] = temp;

	return i+1;
}


void quickSort(int *values, int lower, int upper)
{
	int q;

	if(lower < upper)
	{
		q = Partition(values, lower, upper);
		
		quickSort(values, lower, q-1);
		quickSort(values, q+1, upper);
	}
}


void quickSort(double *values, int lower, int upper)
{
	int q;

	if(lower < upper)
	{
		q = Partition(values, lower, upper);
		
		quickSort(values, lower, q-1);
		quickSort(values, q+1, upper);
	}
}


int binaryFind(int target, int *values, int n)
{
	int low = 0, high = n - 1, mid;

	while(low <= high)
	{
		mid = (low + high) / 2;
		if( target == values[mid] )
			return mid;
		else if (target < values[mid])
			high = mid - 1;
		else
			low = mid + 1;
	}

	cout<<"Error in binaryFind."<<endl;
	exit(1);
}


void randPerm(int n, int *result)
{
	int *original, *sorted, i, flag = 1;
	original = new int[n];
	sorted = new int[n];

	while(flag)
	{
		flag = 0;
		for(i=0; i<n; i++)
		{
			original[i] = rand();
			sorted[i] = original[i];
		}

		quickSort(sorted, 0, n-1);

		for(i=1; i<n; i++)
		{
			if(sorted[i] == sorted[i-1])
			{
				flag = 1;
				break;
			}
		}
	}

	for(i=0; i<n; i++)
		result[i+1] = binaryFind(original[i], sorted, n) + 1;

	delete []original;
	delete []sorted;
}



double innerProduct(myVector *V1, myVector *V2)
{
	double result, *p, *q;
	int i;
	
	if(V1->nRow != V2->nRow)
	{
		cout<<"Unequal dimensionality of vectors to multiplify!\n";
		exit(1);
	}

	result = 0;
	p = V1->data;
	q = V2->data;
	for(i=1; i<=V1->nRow; i++)
	{
		p++;
		q++;
		result += (*p) * (*q);
	}

	return result;
}

// calculates  matrix1 * matrix2
myMatrix myMatMult(myMatrix *matrix1, myMatrix *matrix2)
{
	myMatrix result;
	int i, j, k, show=0;
	double temp, **tempMat;
	
//	cout<<matrix1->nRow<<"*"<<matrix1->nColumn<<" times "<<matrix2->nRow<<"*"<<matrix2->nColumn<<endl;
//	cout.flush();
	if(matrix1->nColumn != matrix2->nRow)
	{
		cout<<"Two matrices are not comfortable for multiplication myMatMult!\n";
		exit(1);
	}

	if(matrix1->nColumn * 1.0 * matrix1->nRow * 1.0 * matrix2->nColumn > 2e9)
		show = 1;

	tempMat = new double *[matrix2->nColumn + 1];
	for(i=1; i<=matrix2->nColumn; i++)
		tempMat[i] = new double[matrix2->nRow + 1];
	for(i=1; i<=matrix2->nColumn; i++)
		for(j=1; j<=matrix2->nRow; j++)
			tempMat[i][j] = matrix2->data[j][i];

	result.nRow = matrix1->nRow;
	result.nColumn = matrix2->nColumn;

	result.data = new double *[result.nRow + 1];
	for(i=1; i<=result.nRow; i++)
		result.data[i] = new double[result.nColumn + 1];

	for(i=1; i<=result.nRow; i++)
	{
		for(j=1; j<=result.nColumn; j++)
		{
			temp = 0;
			for(k=1; k<=matrix1->nColumn; k++)
				temp += matrix1->data[i][k] * tempMat[j][k];
			result.data[i][j] = temp;
		}
		if(show && i % 100 == 0)
		{
			cout<<i<<"  ";
			cout.flush();
		}
	}
//	cout<<endl;
//	cout.flush();
	for(i=1; i<=matrix2->nColumn; i++)
		delete []tempMat[i];
	delete []tempMat;

	return result;		
}


// compute matrix1^T * matrix2
myMatrix myMatTransMult(myMatrix *matrix1, myMatrix *matrix2)
{
	myMatrix result;
	int i, j, k, show=0;
	double temp, **tempMat, **tempMat2;
	
//	cout<<matrix1->nRow<<"*"<<matrix1->nColumn<<" ^T times "<<matrix2->nRow<<"*"<<matrix2->nColumn<<endl;

	if(matrix1->nRow != matrix2->nRow)
	{
		cout<<"Two matrices are not comfortable for trans-multiplication!\n";
		exit(1);
	}

	if(matrix1->nColumn * 1.0 * matrix1->nRow * 1.0 * matrix2->nColumn > 2e9)
		show = 1;
	tempMat = new double *[matrix1->nColumn + 1];
	for(i=1; i<=matrix1->nColumn; i++)
		tempMat[i] = new double[matrix1->nRow + 1];
	for(i=1; i<=matrix1->nColumn; i++)
		for(j=1; j<=matrix1->nRow; j++)
			tempMat[i][j] = matrix1->data[j][i];

	tempMat2 = new double *[matrix2->nColumn + 1];
	for(i=1; i<=matrix2->nColumn; i++)
		tempMat2[i] = new double[matrix2->nRow + 1];
	for(i=1; i<=matrix2->nColumn; i++)
		for(j=1; j<=matrix2->nRow; j++)
			tempMat2[i][j] = matrix2->data[j][i];

	result.nRow = matrix1->nColumn;
	result.nColumn = matrix2->nColumn;

	result.data = new double *[result.nRow + 1];
	for(i=1; i<=result.nRow; i++)
		result.data[i] = new double[result.nColumn + 1];

	for(i=1; i<=result.nRow; i++)
	{
		for(j=1; j<=result.nColumn; j++)
		{
			temp = 0;
			for(k=1; k<=matrix1->nRow; k++)
				temp += tempMat[i][k] * tempMat2[j][k];
			result.data[i][j] = temp;
		}
		if(show && i % 100 == 0 )
		{
			cout<<i<<"  ";
			cout.flush();
		}
	}
//	cout<<endl;
//	cout.flush();
	for(i=1; i<=matrix1->nColumn; i++)
		delete []tempMat[i];
	delete []tempMat;
	for(i=1; i<=matrix2->nColumn; i++)
		delete []tempMat2[i];
	delete []tempMat2;

	return result;			
}


// calculates (vector^T * matrix)^T, or matrix^T * vector
// input vector is n*1
myVector VecTransMultMat(myVector *vector, myMatrix *matrix)
{
	myVector result;
	int i, k;
	double temp;
	
	if(matrix->nRow != vector->nRow)
	{
		cout<<"The transposed vector and matrix are not comfortable for multiplication!\n";
		exit(1);
	}

	result.nRow = matrix->nColumn;
	result.data = new double[result.nRow + 1];

	for(i = 1; i <= result.nRow; i++)
	{
		temp = 0;
		for(k=1; k<=matrix->nRow; k++)
			temp += vector->data[k] * matrix->data[k][i];
		result.data[i] = temp;
	}

	return result;		
}


myVector MatMultVec(myMatrix *matrix, myVector *vector)
{
	myVector result;
	int i, j;
	double temp;

	if(matrix->nColumn != vector->nRow)
	{
		cout<<"Matrix nColumn != vector nRow.  Can't multiply in MatMultVec!\n";
		exit(1);
	}
//	cout<<matrix->nRow<<"*"<<matrix->nColumn<<" times "<<vector->nRow<<endl;
	result.nRow = matrix->nRow;
	result.data = new double[result.nRow + 1];

	for(i=1; i <= matrix->nRow; i++)
	{
		temp = 0;
		for(j=1; j<=matrix->nColumn; j++)
			temp += matrix->data[i][j] * vector->data[j];
		result.data[i] = temp;
	}

	return result;
}


myVector MatTransMultVec(myMatrix *matrix, myVector *vector)
{
	myVector result;
	int i, j;
	double temp;

	if(matrix->nRow != vector->nRow)
	{
		cout<<"Matrix row != vector nRow.  Can't multiply in MatTransMultVec!\n";
		exit(1);
	}
//	cout<<matrix->nRow<<"*"<<matrix->nColumn<<" times "<<vector->nRow<<endl;
	result.nRow = matrix->nColumn;
	result.data = new double[result.nRow + 1];

	for(i=1; i <= matrix->nColumn; i++)
	{
		temp = 0;
		for(j=1; j<=matrix->nRow; j++)
			temp += matrix->data[j][i] * vector->data[j];
		result.data[i] = temp;
	}

	return result;
}


void initMatrix(myMatrix *matrix, int nRow, int nColumn)
{
	int i;

	matrix->nColumn = nColumn;
	matrix->nRow = nRow;

	matrix->data = new double *[nRow + 1];
	for(i=1; i <= nRow; i++)
		matrix->data[i] = new double[nColumn + 1];
}


void initVector(myVector *vector, int dimension)
{
	vector->nRow = dimension;
	vector->data = new double[dimension + 1];
}


void permMatrix(myMatrix *store_B, int *realIndex1, int *realIndex2, myMatrix *B)
{
	int i, j;

/*
	if(realIndex1->nRow != store_B->nRow || realIndex2->nRow != store_B->nColumn)
	{
		cout<<"Unable to permutation the matrix due to incorrect dimension.\n";
		exit(1);
	}
*/
	
	initMatrix(B, store_B->nRow, store_B->nColumn);

	for(i=1; i <= store_B->nRow; i++)
		for(j=1; j <= store_B->nColumn; j++)
			B->data[i][j] = store_B->data[realIndex1[i]][realIndex2[j]];
}


void permVector(myVector *store_V, int *realIndex1, myVector *V)
{
	int i;

	initVector(V, store_V->nRow);

	for(i=1; i <= store_V->nRow; i++)
		V->data[i] = store_V->data[realIndex1[i]];
}


void clearMatrix(myMatrix *matrix)
{
	delete []matrix->data;
}

void clearVector(myVector *vector)
{
	delete []vector->data;
}


void printVector(myVector *vector)
{
	int i;

	for(i=1; i <= vector->nRow; i++)
		cout<<vector->data[i]<<endl;
}



void printMatrix(myMatrix *matrix)
{
	int i, j;

	for(i=1; i <= matrix->nRow; i++)
	{
		for(j=1; j<=matrix->nColumn; j++)
			cout<<matrix->data[i][j]<<"  ";
		cout<<endl;
	}

}


//  Integers in seenClass are free by the dataset, not necessarily starting from 1/0.
//  Any integer is allowed.
int findClass(int label, int *seenLabel, int curNumClass)
{
	int i;

	for(i=1; i <= curNumClass; i++)
		if(seenLabel[i] == label)
			return i;
	return curNumClass + 1;
}


void buildClassList(char *filename, int **paraLabels, int *N, int *numClass, int ***paraClassList)
{
	int **classList, *classCount, i, *seenLabel, curNumClass, curClass, *labels;	
	fstream input, input1;

	*N = 0;
	input.open(filename, ios::in);
	input >> i;	
	while(!input.eof())
	{
		(*N) ++;
		input >> i;		
	}
	input.close();

	labels = new int[*N + 1];
	input1.open(filename, ios::in);	

	for(i=1; i <= *N; i++)
		input1 >> labels[i];

	input1.close();

	curNumClass = 0;
	classCount = new int[*N + 1];
	seenLabel = new int[*N + 1];

	for(i=1; i <= *N; i++)
	{
		curClass = findClass(labels[i], seenLabel, curNumClass);
		if(curClass > curNumClass)
		{
			curNumClass ++;
			seenLabel[curNumClass] = labels[i];
			classCount[curNumClass] = 1;
		}
		else
			classCount[curClass] ++;
	}

	// now we know that there are 
	// curNumClass classes
	// classCount[i] stores the number of items for class i
	classList = new int *[curNumClass + 1];
	for(i=1; i <= curNumClass; i++)
	{
		classList[i] = new int[classCount[i] + 1];
		classList[i][0] = 0;
	}

	for(i=1; i <= *N; i++)
	{
		curClass = findClass(labels[i], seenLabel, curNumClass);
		classList[curClass][0] ++;
		classList[curClass][classList[curClass][0]] = i;
		
		// convert label[i] to its corresponding class
		labels[i] = curClass;
	}	

	*paraClassList = classList;
	*paraLabels = labels;
	*numClass = curNumClass;
	
	delete []classCount;
	delete []seenLabel;
}


int updatePicked(int *starts, int *ends, int i, int numClass)
{
	int j;
	for(j=1; j<=numClass; j++)
		if(starts[j] <= i && i <= ends[j])
			return j;

	cout<<"Error in rand pick."<<endl;
	exit(1);
}

// index from 1, range from 1
void randSplit(int total, int **classList, int numClass, int *perm, 
			   int *numPicked, int *realIndex, int base)
{
	int i, j, curTrain, N, curTest;

	int *starts, *ends;

	N = 0;
	for(i=1; i<=numClass; i++)
		N += classList[i][0];

	starts = new int[numClass + 1];
	ends = new int[numClass + 1];

	starts[1] = 1;
	ends[1] = classList[1][0] - 1;
	for(i=2; i<=numClass; i++)
	{
		starts[i] = ends[i-1]+1;
		ends[i] = starts[i] + classList[i][0] - 2;
	}

	for(i=1; i <= numClass; i++)
		numPicked[i] = base;

	for(i=0; i<total - numClass * base; i++)
	{
		j = rand() % (N - numClass) + 1;
		numPicked[updatePicked(starts, ends, j, numClass)] ++;
	}

	curTrain = 1;
	curTest = total + 1;

	cout<<"numPicked[1~"<<numClass<<"] = ";
	for(i=1; i <= numClass; i++)
	{ 
		randPerm(classList[i][0], perm);
		cout<<numPicked[i]<<",  ";
		for(j=1; j <= numPicked[i]; j++)
			realIndex[curTrain++] = classList[i][perm[j]];
		for(; j <= classList[i][0]; j++)
			realIndex[curTest++] = classList[i][perm[j]];
	}

	cout<<endl;
	delete []starts;
	delete []ends;
}


void getEnvVar(char *flag, char *directory, char *result)
{
	char command[400];
	fstream input;
	struct timeb tp;
	int suffix;

	ftime(&tp);
	suffix = int(tp.time * 1000 + tp.millitm) % 1000000;

	sprintf(command, "echo $%s >> %s/envVar_%d.dat", flag, directory, suffix);
	system(command);

	sprintf(command, "%s/envVar_%d.dat", directory, suffix);
	input.open(command, ios::in);
	input.getline(result, 400);
	input.close();
	
	sprintf(command, "rm %s/envVar_%d.dat", directory, suffix);
	system(command);
}


void buildRealIndex(int *message, int *realIndex, int mynode, int numClass, int N)
{
	int *numPicked = message;
	int **startIndex = new int*[numClass+1];
	int i, j, current;
	
	startIndex[1] = message + numClass;
	for(i=2; i<=numClass; i++)
		startIndex[i] = startIndex[i-1] + message[i-2];

	current = 1;

	for(i=1; i <= message[mynode - 1]; i++)
		realIndex[current ++] = *(startIndex[mynode] + i - 1);

	for(i=1; i < mynode; i++)
		for(j=1; j <= message[i-1]; j++)
			realIndex[current++] = *(startIndex[i] + j - 1);

	for(i = mynode + 1; i <= numClass; i ++)
		for(j=1; j<=numPicked[i-1]; j++)
			realIndex[current++] = *(startIndex[i] + j - 1);
	
	// finally append test data	
	for(; current <= N; current++)
		realIndex[current] = message[current + numClass - 1];
	
	delete []startIndex;
}


int isOdd(int i)
{
	if(i==2 || i==4 || i==6 || i==7 || i==9)
		return 1;
	return 0;
}


void randSplitOddEven(int total, int **classList, int numClass, int *perm, 
			   int *numPicked, int *realIndex)
{
	int i, j, N, nOdd, nEven, oriOdd, oriEven;
	int curTrainOdd, curTrainEven, curTestOdd, curTestEven;

	if(numClass != 10)
	{
		cout<<"Error, numClass must be 10."<<endl;
		exit(1);
	}
//	cout<<"total = "<<total<<endl;

	int *starts, *ends;

	N = 0;
	for(i=1; i<=numClass; i++)
		N += classList[i][0];

	starts = new int[numClass + 1];
	ends = new int[numClass + 1];

	starts[1] = 1;
	ends[1] = classList[1][0] - 1;
	for(i=2; i<=numClass; i++)
	{
		starts[i] = ends[i-1]+1;
		ends[i] = starts[i] + classList[i][0] - 2;
	}

	for(i=1; i <= numClass; i++)
		numPicked[i] = 1;

	for(i=0; i<total - numClass; i++)
	{
		j = rand() % (N - numClass) + 1;
		numPicked[updatePicked(starts, ends, j, numClass)] ++;
	}

	oriOdd = oriEven = nOdd = nEven = 0;
	for(i=1; i<=numClass; i++)
	{
		if( isOdd(i) == 1 )
		{
			nOdd += numPicked[i];
			oriOdd += classList[i][0];
		}
		else
		{
			nEven += numPicked[i];
			oriEven += classList[i][0];
		}
	}

	curTrainOdd = 1;
	curTrainEven = nOdd + 1;
	curTestOdd = nOdd + nEven + 1;
	curTestEven = curTestOdd + oriOdd - nOdd;

	for(i=1; i <= numClass; i++)
	{
		randPerm(classList[i][0], perm);
		cout<<numPicked[i]<<",  ";
		if(isOdd(i) == 1)
		{
			for(j=1; j <= numPicked[i]; j++)
				realIndex[curTrainOdd++] = classList[i][perm[j]];
			for(; j <= classList[i][0]; j++)
				realIndex[curTestOdd++] = classList[i][perm[j]];
		}
		else
		{
			for(j=1; j <= numPicked[i]; j++)
				realIndex[curTrainEven++] = classList[i][perm[j]];
			for(; j <= classList[i][0]; j++)
				realIndex[curTestEven++] = classList[i][perm[j]];
		}
	}

	cout<<endl;
	
	numPicked[1] = nOdd;
	numPicked[2] = nEven;

	delete []starts;
	delete []ends;
}


/* Get counts from file in order to allocate memory
 *   filename - name of file containing instances 
 *   lines - the number of data lines (first line is number of classes)
 *   maxLine - max number of characters in a line
 *   maxFeature - max number features in a line plus 1 (not the maximum feature index)
 */
void getCounts(char *filename, long *lines, long *maxLine, long *maxFeature)
{
	FILE *fid;
	int tempChar;
	char currChar;
	int currCharCt = 0;
	int currFeature = 0;
	if ((fid=fopen(filename, "r")) == NULL){
		perror(filename);
		exit(1);
	}

	(*lines) --;
	while ((tempChar = getc(fid))!= EOF){
		currChar = (char) tempChar;
		currCharCt++;
		if (isspace(tempChar)){
			currFeature++;
		}
		if (currChar == '\n'){
			(*lines)++;
			if (currCharCt > (*maxLine))
				*maxLine = currCharCt;
			if (currFeature > (*maxFeature))
				*maxFeature = currFeature;
			currCharCt = 0;
			currFeature = 0;
		}
	}
	fclose(fid);
}



//  This function is replaced by transReadItems()
//  Almost no change has been made to the function

/* Read in instances from file
 *   filename - name of file containing instances
 *   set - items read in are stored here
 *   numClass - number of classification classes in this problem
 *   maxIndex - largest feature index
 *   avgNorm - average norm over all instances
 *   normalize - normalize instances to length 1 if set to nonzero value
 *   maxItem - from getCounts
 *   maxLine - from getCounts
 *   maxFeature - from getCounts
*/
void readItems(char *filename, ITEMSET *set, int *numClass, long *maxIndex, 
			   double *avgNorm, int normalize, long maxItem, long maxLine, long maxFeature)
{
	FILE *fid;
	char *currLine;
	long *currFeas;
	double *currVals;
	int currLabel;
	int currNum;
	int currItem = 0;
	double sumsqred;
	double maxNorm = 0;
	FEATURE *feaPt;
	int i, tempint, blankLine;
	int lineID;

	*maxIndex =0;
	blankLine = 0;
	lineID = 0;

	if((fid = fopen (filename, "r")) == NULL){ 
		perror (filename); 
		exit (1); 
	}

	*avgNorm = 0;
	set->items = new ITEM[maxItem];
	set->numLabeled = 0;

	/* Allocate enough memory to do reading in */
	currLine = new char[maxLine+1];
	currFeas = new long[maxLine];
	currVals = new double[maxLine];

	/* Read in number of classes */
	fscanf(fid, "%d\n", &tempint);
	*numClass = tempint;

	while((!feof(fid)) && fgets(currLine,(int)(maxLine+1),fid)) 
	{
		if(processLine(currLine, &currLabel, currFeas, currVals, &currNum) != 0) {
			printf("Error reading item %d!\n",currItem);
			exit(1);
		}

		lineID ++;

		feaPt = new FEATURE[currNum];
		sumsqred = 0;
		for (i=0;i<currNum;i++){
			feaPt[i].num = currFeas[i];
			feaPt[i].val = currVals[i];
			sumsqred += currVals[i] * currVals[i];

			if ((*maxIndex)<currFeas[i]) {			// including unlabeled data
				*maxIndex = currFeas[i];
			}
		}

		set->items[currItem].iclass = currLabel;
		set->items[currItem].featureNum = currNum;
		set->items[currItem].norm = sqrt(sumsqred);

		*avgNorm += set->items[currItem].norm;
		if (maxNorm < set->items[currItem].norm)
			maxNorm = set->items[currItem].norm;
		if (normalize != 0){
			for (i=0;i<currNum;i++){
				feaPt[i].val /= set->items[currItem].norm;
			}
		}
		set->items[currItem].features = feaPt;
		currItem++;
	}

	fclose(fid);
	delete []currLine;
	delete []currFeas;
	delete []currVals;
	set->num = currItem;
	*avgNorm /= set->num;

	cout<<"----------------------------------------------------------\nStatistics for data read by readItems() from "<<filename<<":"<<endl;
	cout<<"Total Item: "<<currItem<<endl;
	cout<<"Total Unlabeled Item: "<<currItem - set->numLabeled<<endl;
	cout<<"Total Labeled Item: "<<set->numLabeled<<endl;
	cout<<"Max Index: "<<(*maxIndex)<<"   Max Line: "<<maxLine<<endl;
}


/* Process a single line
 *   line - line to be processed
 *   label - class label
 *   features - array of feature indices
 *   values - array of corresponding feature values
 *   length - number of features processed
 *
 *   Returns 0 if processed correctly
 */
int processLine(char *line, int *label, long *features, double *values, int *length)
{
	int currPos = 0;
	int tempint;
	long templong;
	double tempdouble;
	if(sscanf(line,"%d",label) == EOF) return(0);
	while(isspace((int)line[currPos]))
		currPos++;
	while(!isspace((int)line[++currPos]));
	*length = 0;

	while (((tempint = sscanf(line+currPos,"%ld:%lf",&templong,&tempdouble)) != EOF)) {
		if(tempint != 2) {return 0;}
		if(templong<1) { 
			perror ("Feature numbers must be larger or equal to 1!!!\n"); 
			printf("LINE: %s\n",line);
			exit(1); 
		}

		if((*length>0) && (features[*length-1] >= templong)) {
			perror ("Features must be in increasing order!!!\n"); 
			printf("LINE: %s\n",line);
			exit(1); 
		}

		while(isspace((int)line[currPos++]));
		while(!isspace((int)line[++currPos]));

		features[*length] = templong;
		values[*length] = tempdouble;
		(*length)++;
	}
	return 0;
}
  

// labels[i] are from 1 to C
// classList[curClass][0] is the number of curClass' instance
void buildClassListHyper(AppCtx *user, char *filename)
{
	int **classList, *classCount, i, *seenLabel, curNumClass, curClass, *labels;
	int N = user->set->num;
	char costFileName[MaxStrLen], *cur;

	user->N = user->set->num;
	labels = new int[N + 1];

	for(i=1; i <= N; i++)
		labels[i] = user->set->items[i-1].iclass;

	curNumClass = 0;
	classCount = new int[N + 1];
	seenLabel = new int[N + 1];

	for(i=1; i <= N; i++)
	{
		curClass = findClass(labels[i], seenLabel, curNumClass);
		if(curClass > curNumClass)
		{
			curNumClass ++;
			seenLabel[curNumClass] = labels[i];
			classCount[curNumClass] = 1;
		}
		else
			classCount[curClass] ++;
	}

	// now we know that there are curNumClass classes
	// classCount[i] stores the number of items for class i
	classList = new int *[curNumClass + 1];
	for(i=1; i <= curNumClass; i++)
	{
		classList[i] = new int[classCount[i] + 1];
		classList[i][0] = 0;
	}

	for(i=1; i <= N; i++)
	{
		curClass = findClass(labels[i], seenLabel, curNumClass);
		classList[curClass][0] ++;
		classList[curClass][classList[curClass][0]] = i;
		
		// convert label[i] to its corresponding class
		labels[i] = curClass;
	}	

	user->classList = classList;
	user->labels = labels;

	// now fill in the cost 
	user->label2cost = new double[curNumClass + 1];

	if(user->balance == 0)
		for(i=1; i<=curNumClass; i++)
			user->label2cost[i] = 1;
	else
	{
		strcpy(costFileName, filename);
		cur = costFileName + strlen(filename) - 1;
		while(*cur != '.')
			cur --;
		strcpy(cur+1, "cost");

		fstream input;
		input.open(costFileName, ios::in);
		if(input.is_open() == 0)
		{
			cout<<"Error in openning "<<costFileName<<"."<<endl;
			exit(1);
		}
		for(i=1; i<=curNumClass; i++)
		{
			input >> curClass;
			curClass = findClass(curClass, seenLabel, curNumClass);
			input >> user->label2cost[curClass];
			if(input.eof())
			{
				cout<<"Error in reading cost file.  Insufficient lines."<<endl;
				exit(1);
			}
		}
		input.close();
	}

	delete []classCount;
	delete []seenLabel;

	user->bal_L = new double[user->L+1];
	user->bal_U = new double[user->U+1];	
}


// the index and value of realIndex[] both start from 1
void genRealIndex(AppCtx *user, int L, int base=1)
{	
	int *perm, *numPicked, i;

	perm = new int[user->N + 1];
	numPicked = new int[3];

	randSplit(L, user->classList, 2, perm, numPicked, user->realIndex, base);

	user->nPos = numPicked[1];
	user->nNeg = numPicked[2];

	delete []perm;
	delete []numPicked;

	for(i=1; i<=L; i++)
	{
		if(user->labels[user->realIndex[i]] < 1.5)
			user->fl->data[i] = 1;
		else
			user->fl->data[i] = 0;

		user->bal_L[i] = user->label2cost[user->labels[user->realIndex[i]]];
	}

	for(; i<=user->N; i++)
	{
		if(user->labels[user->realIndex[i]] > 1.5)
			user->fref->data[i-L] = 0;
		else
			user->fref->data[i-L] = 1;

		user->bal_U[i-L] = user->label2cost[user->labels[user->realIndex[i]]];
	}
}


// the index and value of realIndex[] both start from 1
void fillIndex(AppCtx *user)
{	
	int i, numLabeledPos, numLabeledNeg, numUnlabeledPos, numUnlabeledNeg;

	// just need to fill the user->realIndex array
	numLabeledPos = numLabeledNeg = numUnlabeledPos = numUnlabeledNeg = 0;
	for(i=1; i<=user->N; i++)
	{
		if(user->labels[i] < 1.5)		// positive
		{
			if(user->typeTag[i] == 1)	// labeled
				numLabeledPos ++;
			else						// unlabeled
				numUnlabeledPos ++;
		}
		else							// negative
		{
			if(user->typeTag[i] == 1)	// labeled
				numLabeledNeg ++;
			else						// unlabeled
				numUnlabeledNeg ++;
		}
	}

	user->nPos = numLabeledPos;
	user->nNeg = numLabeledNeg;
	user->L = numLabeledPos + numLabeledNeg;

	numUnlabeledNeg = numLabeledPos + numUnlabeledPos + numLabeledNeg + 1;
	numUnlabeledPos = numLabeledPos + numLabeledNeg + 1;
	numLabeledNeg = numLabeledPos + 1;
	numLabeledPos = 1;

	for(i=1; i<=user->N; i++)
	{
		if(user->labels[i] < 1.5)		// positive
		{
			if(user->typeTag[i] == 1)	// labeled
			{
				user->realIndex[numLabeledPos] = i;
				numLabeledPos ++;
			}
			else						// unlabeled
			{
				user->realIndex[numUnlabeledPos] = i;
				numUnlabeledPos ++;
			}
		}
		else							// negative
		{
			if(user->typeTag[i] == 1)	// labeled
			{
				user->realIndex[numLabeledNeg] = i;
				numLabeledNeg ++;
			}
			else						// unlabeled
			{
				user->realIndex[numUnlabeledNeg] = i;
				numUnlabeledNeg ++;
			}
		}
	}

	for(i=1; i<=user->L; i++)
	{
		if(user->labels[user->realIndex[i]] < 1.5)
			user->fl->data[i] = 1;
		else
			user->fl->data[i] = 0;
		user->bal_L[i] = user->label2cost[user->labels[user->realIndex[i]]];
	}

	for(; i<=user->N; i++)
	{
		if(user->labels[user->realIndex[i]] > 1.5)
			user->fref->data[i-user->L] = 0;
		else
			user->fref->data[i-user->L] = 1;
		user->bal_U[i-user->L] = user->label2cost[user->labels[user->realIndex[i]]];
	}
}


void simplifyFilename(char *filename)
{
	int length, i, current, preSlash;

	length = strlen(filename);
	current = 0;
	preSlash = 0;

	for(i=0; i<length; i++)
	{
		if(filename[i] == '/' || filename[i] == '\\')
		{
			if(preSlash == 0)
			{
				preSlash = 1;
				filename[current++] = filename[i];
			}
		}
		else
		{
			preSlash = 0;
			filename[current++] = filename[i];
		}
	}
	filename[current] = '\0';
}


// output the result of running the command to a temporary file
//   and copy the output to *retString (will flush its original value)
void runShellCommand(char *command, char *directory, char **retString)
{
	char filename[400], newCommand[400];
	fstream input, input1;
	int length, firstLine, numLine;

	sprintf(filename, "%s/tempCommandResult.dat", directory);
	simplifyFilename(filename);

	sprintf(newCommand, "%s >& %s", command, filename);
	system(newCommand);

	length = 0;
	input.open(filename, ios::in);
	input.getline(newCommand, 400);
	numLine = 0;
	while(!input.eof())
	{
		numLine ++;
		length += strlen(newCommand) + 1;
		input.getline(newCommand, 400);
	}
	input.close();

	if(*retString != NULL)
		delete (*retString);
	*retString = new char[length + numLine + 1];

	strcpy(*retString, "");
	firstLine = 1;

	input1.open(filename, ios::in);
	input1.getline(newCommand, 400);
	while(!input1.eof())
	{
		if(firstLine == 1)
			firstLine = 0;
		else
			strcat(*retString, "\n");
		strcat(*retString, newCommand);
		input1.getline(newCommand, 400);
	}
	input1.close();	

	sprintf(newCommand, "rm %s", filename);
	system(newCommand);
}


inline int myMax(int a, int b)
{
	return a > b ? a : b;
}


Locker::Locker(int _type)
{
	type = _type;
	server = filename = NULL;
	pid = -1;
}


Locker::~Locker()
{
	if(server)
		delete []server;
	if(filename)
		delete []filename;
}


void Locker::setType(int _type)
{
	type = _type;
}


void Locker::setFilename(char *_filename)
{
	if(type == 1 || type == 2)
	{
		filename = new char[strlen(_filename) + 1];
		strcpy(filename, _filename);
		simplifyFilename(filename);
	}
}


void Locker::initLock_file()
{
	if((fileID = open(filename, O_WRONLY | O_CREAT, S_IRWXU)) == -1)
	{
		cout<<"Cannot open "<<filename<<endl;
		exit(1);
	}

	close(fileID);
	fl.l_whence = SEEK_SET;
	fl.l_start  = 0;
	fl.l_len    = 0;
	fl.l_pid    = pid;
}


void Locker::lock_file()
{
	if((fileID = open(filename, O_WRONLY | O_CREAT, S_IRWXU)) == -1)
	{
		cout<<"Cannot open "<<filename<<endl;
		exit(1);
	}

	fl.l_type   = F_WRLCK;
	if (fcntl(fileID, F_SETLKW, &fl) == -1)
	{
		perror("fcntl");
		exit(1);
	}
}


void Locker::unlock_file()
{
	fl.l_type = F_UNLCK;

	if (fcntl(fileID, F_SETLK, &fl) == -1)
	{
		perror("fcntl");
		exit(1);
	}
	close(fileID);
}


void Locker::initLock_sem()
{
	key_t key;
	
	if ((key = ftok(filename, 'J')) == -1) 
	{
		perror("ftok");
		exit(1);
	}

	// get ID of semaphore set created by init_sem.c:
	if ((sem_set_id = semget(key, 1, 0)) == -1) 
	{
		perror("Error in semget called by get_sem_id is");
		exit(1);
	}
}


void Locker::lock_sem()
{
	struct sembuf sb_wait = {0, -1, 0};

	if (semop(sem_set_id, &sb_wait, 1) == -1) 
	{
		perror("Error in semop called by wait is");
		exit(1);
	}
}


void Locker::unlock_sem()
{
	struct sembuf sb_signal = {0, 1, 0};

	if (semop(sem_set_id, &sb_signal, 1) == -1) 
	{
		perror("Error in semop called by signal is");
		exit(1);
	}
}


void Locker::initLock_socket()
{
	server = new char[strlen(getenv("ServerName")) + 1];
	strcpy(server, getenv("ServerName"));

	if ((he = gethostbyname(server)) == NULL) {  // get the host info 
		perror("gethostbyname");
		exit(1);
	}

	port = atoi(getenv("PORT"));
	their_addr.sin_family = AF_INET;    // host byte order 
	their_addr.sin_port = htons(port);  // short, network byte order 
	their_addr.sin_addr = *((struct in_addr *) he->h_addr);
	memset(&(their_addr.sin_zero), '\0', 8);  // zero the rest of the struct 

	sprintf(sok_buf, "-%ld", pid);
	lenpid = strlen(sok_buf) - 1;
}


void Locker::lock_socket()
{
	int sockfd, numbytes;  

	if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) 
	{
	    perror("socket");
	    exit(1);
	}

	if (connect(sockfd, (struct sockaddr *)&their_addr, sizeof(struct sockaddr)) == -1) 
	{
	    perror("connect");
	    exit(1);
	}

	if (send(sockfd, sok_buf + 1, lenpid, 0) == -1)
	{
		perror("send");
		exit(1);
	}

	if ((numbytes = recv(sockfd, sok_buf2, MaxReplyLen, 0)) == -1) 
	{
	    perror("recv");
	    exit(1);
	}

	close(sockfd);
}


void Locker::unlock_socket()
{
	int sockfd, numbytes;  

	if ((sockfd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
	    perror("socket");
	    exit(1);
	}

	if (connect(sockfd, (struct sockaddr *)&their_addr, sizeof(struct sockaddr)) == -1) 
	{
	    perror("connect");
	    exit(1);
	}

	if (send(sockfd, sok_buf, lenpid + 1, 0) == -1)
	{
		perror("send");
		exit(1);
	}

	if ((numbytes = recv(sockfd, sok_buf2, MaxReplyLen, 0)) == -1) 
	{
	    perror("recv");
	    exit(1);
	}

	close(sockfd);
}


void Locker::initLock()
{
	pid = getpid();
	if(type == 1)
		initLock_file();
	else if (type == 2)
		initLock_sem();
	else if (type == 3)
		initLock_socket();
}


void Locker::lock()
{	
	if(pid < 0)
	{
		cout<<"Locker has not been initialized.\n";
		exit(1);
	}
	if(type == 1)
		lock_file();
	else if (type == 2)
		lock_sem();
	else if (type == 3)
		lock_socket();

}


void Locker::unlock()
{
	if(pid < 0)
	{
		cout<<"Locker has not been initialized.\n";
		exit(1);
	}
	usleep(100000);
	if(type == 1)
		unlock_file();
	else if (type == 2)
		unlock_sem();
	else if (type == 3)
		unlock_socket();
}


