#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <iostream>
#include <time.h>
#include <sys/timeb.h>
#include <unistd.h>
#include <math.h>
#include <fstream>
#include <netdb.h>
#include <sys/types.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/ipc.h>
#include <sys/sem.h>

using namespace std;

const int MaxStrLen = 400;
const int maxBatch = 10000;
const int maxExp = 1000;
const int MaxTaskPerBatch = 10000;

typedef struct myMatrix
{
	int nColumn, nRow;
	double **data;
}myMatrix;


typedef struct myVector
{
	int nRow;
	double *data;
}myVector;


typedef struct feature {
	int num;		/* Feature number */
 	double val;	/* Feature value */
} FEATURE;


/* item is a vector of features and its class */
typedef struct item { 
  int iclass;
  int featureNum;
  double norm;
  FEATURE *features;
} ITEM;


typedef struct itemset {
	int num;			/* Number of items in the set */
	int testNum;		/* number of testing data */
	int numLabeled;		/* chaileon 1907: Number of labeled items in the set */
	ITEM *items;
} ITEMSET;


class AppCtx {
public:
	// problem parameters
	int         N, L, U, nPos, nNeg, oriPos, oriNeg, feaNum, current, curFeature, standalone, balance;
	int			*realIndex, **classList, *labels, repTime, finalTrial, cv, *typeTag;
	double		preAccuracy, preAccuracy2, postAccuracy, postAccuracy2, variance, *label2cost, *bal_U, *bal_L, fcn;
	myVector    *fl, *f, *fref;
	myMatrix	*weight;
	
	// Working space
	ITEMSET		*set;
	time_t		timeRec;

	char		*directory;		// working directory
};


////////////////         For reading parameters        //////////////////
int getParaInt(int *result, int argc, char *args[], char *para, int mode);
int getParaLong(long *result, int argc, char *args[], char *para, int mode);
int getParaFloat(double *result, int argc, char *args[], char *para, int mode);
int getParaString(char **result, int argc, char *args[], char *para, int mode);
void *checkMalloc(long int size);
void compressDir(char *filename);
int exist(char *filename);


////////////////         For random permutation         ////////////////
void quickSort(int *values, int lower, int upper);
void quickSort(double *values, int lower, int upper);
int  Partition(double *values, int lower, int upper);
int  Partition(int *values, int lower, int upper);
int  binaryFind(int target, int *values, int n);
void buildClassList1(char *filename, int **paraLabels, int *N, int *numClass, int ***paraClassList);
void getEnvVar(char *flag, char *directory, char *result);
void buildRealIndex(int *message, int *realIndex, int mynode, int numClass, int N);
int  isOdd(int i);
void randSplitOddEven(int total, int **classList, int numClass, int *perm, 
			   int *numPicked, int *realIndex);

void randPerm(int n, int *result);
void randSplit(int total, int **classList, int numClass, int *perm, int *numPicked, int *realIndex, int base=1);
void buildClassList(char *filename, int **paraLabels, int *N, int *numClass, int ***paraClassList);
void buildClassListHyper(AppCtx *user, char *filename);
int  findClass(int label, int *seenLabel, int curNumClass);
void genRealIndex(AppCtx *user, int L, int base);
void fillIndex(AppCtx *user);


/////////////////        For matrix/vecotr manipulation         /////////////////
myVector MatMultVec(myMatrix *matrix, myVector *vector);
myVector MatTransMultVec(myMatrix *matrix, myVector *vector);
myVector VecTransMultMat(myVector *vector, myMatrix *matrix);
myMatrix myMatMult(myMatrix *matrix1, myMatrix *matrix2);
double innerProduct(myVector *V1, myVector *V2);
myMatrix myMatTransMult(myMatrix *matrix1, myMatrix *matrix2);
void clearMatrix(myMatrix *matrix);
void permMatrix(myMatrix *store_B, int *realIndex1, int *realIndex2, myMatrix *B);
void permVector(myVector *store_V, int *realIndex1, myVector *V);
void initVector(myVector *vector, int dimension);
void initMatrix(myMatrix *matrix, int nColumn, int nRow);
void clearVector(myVector *vector);
void printVector(myVector *vector);
void printMatrix(myMatrix *matrix);


/////////////////       For reading the data file          /////////////////
void getCounts(char *filename, long *lines, long *maxLine, long *maxFeature);
void readItems(char *filename, ITEMSET *set, int *numClass, long *maxIndex, 
			   double *avgNorm, int normalize, long maxItem, long maxLine, long maxFeature);
int processLine(char *line, int *label, long *features, double *values, int *length);


/////////////////       Shell level utilities		///////////////////
void simplifyFilename(char *filename);
void runShellCommand(char *command, char *directory, char **retString);


/////////////////		Locker       ///////////////////
const int MaxReplyLen = 3;
class Locker
{
public:
	int type, fileID, serverId, sem_set_id, lenpid, port;
	long pid;
	char *filename, *server;
	char sok_buf[MaxStrLen], sok_buf2[MaxReplyLen + 1];
	struct flock fl;
	struct hostent *he;
	struct sockaddr_in their_addr; // connector's address information 
public:
	Locker(int _type = 3);
	~Locker();
	void setType(int _type);
	void setFilename(char *_filename);
	void initLock_file();
	void lock_file();
	void unlock_file();
	void initLock_sem();
	void lock_sem();
	void unlock_sem();
	void initLock_socket();
	void lock_socket();
	void unlock_socket();
	void initLock();
	void lock();
	void unlock();
};


/////////////////		Other       ///////////////////
inline int myMax(int a, int b);


